import 'dart:convert';

import 'InAppProperties.dart';
import 'InAppSchema.dart';

class InApp {
  static const String _INAPPID = "inAppId";
  static const String _LAST_VERSION_ID = "lastVersionId";
  static const String _SHOW_ONCE = "showOnce";
  static const String _RENEWAL_TIME = "renewalTime";
  static const String _NUMBER_OF_SHOWS = "numberOfShows";
  static const String _NUMBER_OF_CLICKS = "numberOfClicks";
  static const String _CREATION_DATE = "creationDate";
  static const String _EXPIRED_DATE = "expiredDate";

  static const String _PROPERTIES = InAppProperties.PROPERTIES;
  static const String _SCHEMA = InAppSchema.SCHEMA;

  int? inAppId;
  int? lastVersionId;
  bool? showOnce;
  int? numberOfShows;
  int? numberOfClicks;
  String? creationDate;
  String? expiredDate;

  InAppProperties? properties;
  String? renewalTime;
  InAppSchema? schema;

  /// Constructor of [InApp]
  InApp(Map map) {
    map.forEach((key, value) {
      switch (key) {
        case _INAPPID:
          inAppId = int.parse(value.toString());
          break;
        case _LAST_VERSION_ID:
          lastVersionId = int.parse(value.toString());
          break;
        case _SHOW_ONCE:
          showOnce = value.toString() == "true";
          break;
        case _RENEWAL_TIME:
          renewalTime = value.toString();
          break;
        case _NUMBER_OF_SHOWS:
          numberOfShows = int.parse(value.toString());
          break;
        case _NUMBER_OF_CLICKS:
          numberOfClicks = int.parse(value.toString());
          break;
        case _CREATION_DATE:
          creationDate = value.toString();
          break;
        case _EXPIRED_DATE:
          expiredDate = value.toString();
          break;
        case _PROPERTIES:
          properties = InAppProperties(value);
          break;
        case _SCHEMA:
          schema = InAppSchema(value);
          break;
      }
    });
  }

  /// Conversion InApp to Map
  Map toMap() {
    Map map = Map();
    if (inAppId != null) map[_INAPPID] = inAppId;
    if (lastVersionId != null) map[_LAST_VERSION_ID] = lastVersionId;
    if (showOnce != null) map[_SHOW_ONCE] = showOnce;
    if (renewalTime != null) map[_RENEWAL_TIME] = renewalTime;
    if (numberOfShows != null) map[_NUMBER_OF_SHOWS] = numberOfShows;
    if (numberOfClicks != null) map[_NUMBER_OF_CLICKS] = numberOfClicks;
    if (creationDate != null) map[_CREATION_DATE] = creationDate;
    if (expiredDate != null) map[_EXPIRED_DATE] = expiredDate;
    if (properties != null) map[_PROPERTIES] = properties!.toMap();
    if (schema != null) map[_SCHEMA] = schema!.toMap();
    return map;
  }
}
