import 'dart:convert';
import 'dart:io';

import 'package:indigitall_flutter_plugin/push/models/Push.dart';

class InboxNotification {
  static const MAP_ID = "id";
  static const MAP_SENT_AT = "sentAt";
  static const MAP_STATUS = "status";
  static const MAP_SENDING_ID = "sendingId";
  static const MAP_CAMPAIGN_ID = "campaignId";
  static const MAP_MESSAGE = "message";
  static const MAP_READ = "read";
  static const NOTIFICATIONS = "notifications";

  String? id;
  String? sentAt;
  String? status;
  String? sendingId;
  String? campaignId;
  Push? message;
  bool read = false;

  /// Constructor fo InboxNotification
  ///
  /// Param with Map
  InboxNotification(Map? map) {
    map?.forEach((key, value) {
      if (key == MAP_ID) id = value.toString();
      if (key == MAP_SENT_AT) sentAt = value.toString();
      if (key == MAP_STATUS) status = value.toString();
      if (key == MAP_SENDING_ID) sendingId = value.toString();
      if (key == MAP_MESSAGE) {
        if (Platform.isAndroid) {
          message = new Push(new Map<String, String>.from(json.decode(value)));
        } else {
          //message = new Push(new Map<String, dynamic>.from(value));
        }
      }
      if (key == MAP_READ) read = value.toString() == "true";
    });
  }

  /// Create list of notifications from a string
  ///
  /// Param [String] of inbox notifications
  /// return a list of [InboxNotification]
  static List<InboxNotification> createArray(String inboxNotifications) {
    List<dynamic> list = json.decode(inboxNotifications);
    List<InboxNotification> inboxNotificationList = [];
    list.forEach((element) {
      Map map = new Map();
      if (Platform.isAndroid) {
        map = json.decode(element.toString());
      } else {
        map = element;
      }
      InboxNotification inboxNotification = new InboxNotification(map);
      inboxNotificationList.add(inboxNotification);
    });
    return inboxNotificationList;
  }

  /// Conversion to Map
  Map toMap() {
    Map map = Map();
    if (id != null) map[MAP_ID] = id;
    if (sentAt != null) map[MAP_SENT_AT] = sentAt;
    if (status != null) map[MAP_STATUS] = status;
    if (sendingId != null) map[MAP_SENDING_ID] = sendingId;
    if (message != null) map[MAP_MESSAGE] = message;
    map[MAP_READ] = read;

    return map;
  }
}
