import 'dart:convert';
import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_inappwebview/flutter_inappwebview.dart';
import 'package:indigitall_flutter_plugin/inapp/models/InApp.dart';

import 'package:indigitall_flutter_plugin/push/models/Device.dart';
import 'package:indigitall_flutter_plugin/core/models/ErrorModel.dart';
import 'package:indigitall_flutter_plugin/push/models/Push.dart';
import 'package:indigitall_flutter_plugin/push/models/Topic.dart';

import 'package:indigitall_flutter_plugin/inbox/callbacks/InboxAuthListener.dart';
import 'package:indigitall_flutter_plugin/inbox/models/Inbox.dart';
import 'package:indigitall_flutter_plugin/inbox/models/InboxCounter.dart';
import 'package:indigitall_flutter_plugin/inbox/models/InboxNotification.dart';

class IndigitallFlutterPlugin {
  ///Private action interface to call INIT for native SDK
  static const _ACTION_INIT = "initialize";

  ///Private action interface to call DEVICE ENABLE for native SDK
  static const _ACTION_DEVICE_ENABLE = "deviceEnable";

  ///Private action interface to call DEVICE DISABLE for native SDK
  static const _ACTION_DEVICE_DISABLE = "deviceDisable";

  ///Private action interface to call TOPIC LIST for native SDK
  static const _ACTION_TOPICS_LIST = "topicsList";

  ///Private action interface to call TOPIC SUBSCRIBE for native SDK
  static const _ACTION_TOPICS_SUBSCRIBE = "topicsSubscribe";

  ///Private action interface to call TOPIC UNSUBCRIBE for native SDK
  static const _ACTION_TOPICS_UNSUBSCRIBE = "topicsUnsubscribe";

  ///Private action interface to call EXTERNAL CODE for native SDK
  static const _ACTION_EXTERNALCODE = "setExternalCode";

  ///Private action interface to call SEND CUSTOM EVENT for native SDK
  static const _ACTION_SENDCUSTOMEVENT = "sendCustomEvent";

  ///Private action interface to call GET PUSH for native SDK
  static const _ACTION_GET_PUSH = "getPush";

  ///Private action interface to call LOGIN for native SDK
  static const _ACTION_LOGIN = "logIn";

  ///Private action interface to call LOG OUT for native SDK
  static const _ACTION_LOGOUT = "logOut";

  ///Private action interface to call GET INBOX for native SDK
  static const _ACTION_GET_INBOX = "getInbox";

  ///Private action interface to call GET MESSAGE COUNT for native SDK
  static const _ACTION_GET_MESSAGE_COUNT = "getMessageCount";

  ///Private action interface to call NEXT PAGE for native SDK
  static const _ACTION_GET_NEXT_PAGE = "getNextPage";

  ///Private action interface to call MASSIVE EDIT NOTIFICATION for native SDK
  static const _ACTION_MASSIVE_EDIT =
      "massiveEditNotificationsWithSendingIdsList";

  ///Private action interface to call GET INFO NOTIFICATION for native SDK
  static const _ACTION_GET_INFO_NOTIFICATION =
      "getInfoFromNotificationWithSendingId";

  ///Private action interface to call MODIFY STATUS OF NOTIFICATION for native SDK
  static const _ACTION_MODIFY_STATUS =
      "modifyStatusFromNotificationWithSendingId";

  ///Private action interface to call GET CONFIG OF AUTHENTIFICATION for native SDK
  static const _ACTION_GET_AUTH_CONFIG = "getAuthConfig";

  ///Private action interface to call GET INAPP for native SDK
  static const _ACTION_GET_INAPP = "getInApp";

  ///Private action interface to call TAP INAPP for native SDK
  static const _ACTION_TAP_INAPP = "tapInApp";

  /// Private action interface to call SHOW POP UP
  static const _ACTION_SHOW_POP_UP = "showPopUp";

  /// Private action interface to call SHOW INAPP
  static const _ACTION_SHOW_INAPP = "showInApp";

  ///Private ON INDIGITALL INITIALIZE name to get the value of native SDK callbacks
  static const _CALLBACK_ON_INDIGITALL_INITIALIZED =
      "callbackOnIndigitallInitialized";

  ///Private NEW USER REGISTERED name to get the value of native SDK callbacks
  static const _CALLBACK_ON_NEW_USER_REGISTERED = "callbackOnNewUserRegistered";

  ///Private DEVICE ENABLE name to get the value of native SDK callbacks
  static const _CALLBACK_DEVICE_ENABLE = "callbackDeviceEnabled";

  ///Private DEVICE DISABLE name to get the value of native SDK callbacks
  static const _CALLBACK_DEVICE_DISABLE = "callbackDeviceDisEnabled";

  ///Private TOPIC LIST name to get the value of native SDK callbacks
  static const _CALLBACK_TOPIC_LIST = "callbackTopicList";

  ///Private TOPICS SUBSCRIBE name to get the value of native SDK callbacks
  static const _CALLBACK_TOPIC_SUBSCRIBE = "callbackTopicSubscribe";

  ///Private TOPICS UNSUBSCRIBE name to get the value of native SDK callbacks
  static const _CALLBACK_TOPIC_UNSUBSCRIBE = "callbackTopicUnsubscribe";

  ///Private EXTERNAL CODE name to get the value of native SDK callbacks
  static const _CALLBACK_EXTERNAL_CODE = "callbackExternalCode";

  ///Private SEND CUSTOM EVENT name to get the value of native SDK callbacks
  static const _CALLBACK_SEND_CUSTOM_EVENT = "callbackSendCustomEvent";

  ///Private LOG IN name to get the value of native SDK callbacks
  static const _CALLBACK_LOGIN = "callbackLogin";

  ///Private LOGOUT name to get the value of native SDK callbacks
  static const _CALLBACK_LOGOUT = "callbackLogout";

  ///Private GET PUSH name to get the value of native SDK callbacks
  static const _CALLBACK_GET_PUSH = "callbackgetPush";

  ///Private GET INBOX name to get the value of native SDK callbacks
  static const _CALLBACK_GET_INBOX = "callbackGetInbox";

  ///Private MESSAGE COUNT name to get the value of native SDK callbacks
  static const _CALLBACK_GET_MESSAGE_COUNT = "callbackGetMessageCount";

  ///Private NEXT PAGE name to get the value of native SDK callbacks
  static const _CALLBACK_GET_NEXT_PAGE = "callbackGetNextPage";

  ///Private MASSIVE EDIT NOTIFICATION name to get the value of native SDK callbacks
  static const _CALLBACK_MASSIVE_EDIT = "callbackMassiveEdit";

  ///Private GET INFO NOTIFICATION name to get the value of native SDK callbacks
  static const _CALLBACK_GET_INFO_NOTIFICATION = "callbackGetInfoNotification";

  ///Private MODIFY NOTIFICATION STATUS name to get the value of native SDK callbacks
  static const _CALLBACK_MODIFY_STATUS = "callbackModifyStatus";

  ///Private GET INAPP name to get the value of native SDK callbacks
  static const _CALLBACK_GET_INAPP = "callbackGetInApp";

  ///Private DID DISMISSED name to get the value of native SDK callback
  static const _CALLBACK_DID_DISMISSED = "callbackDidDismissed";

  ///Private DID CLOSED name to get the value of native SDK callback
  static const _CALLBACK_DID_CLOSED = "callbackDidClosed";

  ///Private DID CLICKED name to get the value of native SDK callback
  static const _CALLBACK_DID_CLICKED = "callbackDidClicked";

  ///Private DID SHOW POP-UP name to get the value of native SDK callback
  static const _CALLBACK_SHOW_POPUP = "callbackShowPopUp";

  ///Private DID SHOWTIME FINISHED name to get the value of native SDK callback
  static const _CALLBACK_SHOWTIME_FINISHED = "callbackShowTimeFinished";

  ///Private SHOW INAPP name to get the value of native SDK callback
  static const _CALLBACK_SHOW_INAPP = "callbackShowInApp";

  ///name of the plugin
  static const MethodChannel _indigitall =
      const MethodChannel('indigitall_flutter_plugin');

  /// Test connection with plugin
  ///
  /// Return a [String]
  static Future<String> test() async {
    return await "texto";
  }

  /// Initialization of Indigitall
  ///
  /// Callback onIndigitallInitialized return [Device] when the device is ready to receive notifications from indigitall
  /// Only for Android: Callback onNewUserRegistered [Device] when the device has been registered for the first time,
  /// Throw onErrorInitialized [ErrorModel] if there is some error during the processs
  static void init(
      Map params,
      Function(Device device)? onIndigitallInitialized,
      Function(Device device)? onNewUserRegistered,
      Function(ErrorModel error)? onErrorInitialized) async {
    print("[IND] init");
    params["productName"] = "flutter";
    params["productVersion"] = "0.1.1";
    await _indigitall
        .invokeMapMethod(_ACTION_INIT, params)
        .then((value) => {
              if (value != null)
                {
                  value.forEach((key, valueMap) {
                    if (key == _CALLBACK_ON_INDIGITALL_INITIALIZED) {
                      Device device = new Device(valueMap);
                      if (onIndigitallInitialized != null)
                        onIndigitallInitialized(device);
                    }
                    if (key == _CALLBACK_ON_NEW_USER_REGISTERED) {
                      Device device = new Device(valueMap);
                      if (onNewUserRegistered != null)
                        onNewUserRegistered(device);
                    }
                  })
                }
            })
        .catchError((error) => {
              if (error is PlatformException)
                {
                  if (onErrorInitialized != null)
                    onErrorInitialized(ErrorModel(error.code, error.message))
                }
            });
  }

  /// Enable the Device to receive push notifications
  ///
  /// Callback to get [Device]'s data
  /// Throw [ErrorModel] if there is some error during the processs
  static void deviceEnable(Function(Device device)? onSuccess,
      Function(ErrorModel error)? onError) async {
    _indigitall
        .invokeMapMethod(_ACTION_DEVICE_ENABLE)
        .then((value) => {
              if (value != null)
                {
                  value.forEach((key, valueMap) {
                    if (key == _CALLBACK_DEVICE_ENABLE) {
                      if (onSuccess != null) onSuccess(new Device(valueMap));
                    }
                  })
                }
            })
        .catchError((error) => {
              if (error is PlatformException)
                {
                  if (onError != null)
                    onError(ErrorModel(error.code, error.message))
                }
            });
  }

  /// Disable the Device to receive push notifications
  ///
  /// Callback to get [Device]'s data.
  /// Throw [ErrorModel] if there is some error during the processs
  static void deviceDisable(Function(Device device)? onSuccess,
      Function(ErrorModel error)? onError) async {
    _indigitall
        .invokeMapMethod(_ACTION_DEVICE_DISABLE)
        .then((value) => {
              if (value != null)
                {
                  value.forEach((key, valueMap) {
                    if (key == _CALLBACK_DEVICE_DISABLE) {
                      if (onSuccess != null) onSuccess(new Device(valueMap));
                    }
                  })
                }
            })
        .catchError((error) => {
              if (error is PlatformException)
                {
                  if (onError != null)
                    onError(ErrorModel(error.code, error.message))
                }
            });
  }

  /// Get Topic's list
  ///
  /// Callback to get [Topic]'s list.
  /// Throw [ErrorModel] if there is some error during the processs
  static void topicList(Function(List<Topic> topic)? onSuccess,
      Function(ErrorModel error)? onError) async {
    _indigitall
        .invokeMapMethod(_ACTION_TOPICS_LIST)
        .then((value) => {
              if (value != null)
                {
                  value.forEach((key, valueMap) {
                    if (key == _CALLBACK_TOPIC_LIST) {
                      if (onSuccess != null)
                        onSuccess(Topic.createArray(valueMap));
                    }
                  })
                }
            })
        .catchError((error) => {
              if (error is PlatformException)
                {
                  if (onError != null)
                    onError(ErrorModel(error.code, error.message))
                }
            });
  }

  /// Subscribe to Topics
  ///
  /// topics Array of [Topics] to subscribe
  /// Callback to get [Topic]'s list
  /// Throw [ErrorModel] if there is some error during the processs
  static void topicSubscribe(
      List<String> params,
      Function(List<Topic> topic)? onSuccess,
      Function(ErrorModel error)? onError) async {
    _indigitall
        .invokeMapMethod(_ACTION_TOPICS_SUBSCRIBE, params)
        .then((value) => {
              if (value != null)
                {
                  value.forEach((key, valueMap) {
                    if (key == _CALLBACK_TOPIC_SUBSCRIBE) {
                      if (onSuccess != null)
                        onSuccess(Topic.createArray(valueMap));
                    }
                  })
                }
            })
        .catchError((error) => {
              if (error is PlatformException)
                {
                  if (onError != null)
                    onError(ErrorModel(error.code, error.message))
                }
            });
  }

  /// Unsubscribe to Topics
  ///
  /// topics Array of [Topics] to unsubscribe
  /// Callback to get [Topic]'s list
  /// Throw [ErrorModel] if there is some error during the processs
  static void topicUnsubscribe(
      List<String> params,
      Function(List<Topic> topic)? onSuccess,
      Function(ErrorModel error)? onError) async {
    _indigitall
        .invokeMapMethod(_ACTION_TOPICS_UNSUBSCRIBE, params)
        .then((value) => {
              if (value != null)
                {
                  value.forEach((key, valueMap) {
                    if (key == _CALLBACK_TOPIC_UNSUBSCRIBE) {
                      if (onSuccess != null)
                        onSuccess(Topic.createArray(valueMap));
                    }
                  })
                }
            })
        .catchError((error) => {
              if (error is PlatformException)
                {
                  if (onError != null)
                    onError(ErrorModel(error.code, error.message))
                }
            });
  }

  /// Set external code
  ///
  /// params [String] with the code
  /// Callback to get [Device] info
  /// Throw [ErrorModel] if there is some error during the processs
  static void setExternalCode(String params, Function(Device device)? onSuccess,
      Function(ErrorModel error)? onError) async {
    _indigitall
        .invokeMapMethod(_ACTION_EXTERNALCODE, params)
        .then((value) => {
              if (value != null)
                {
                  value.forEach((key, valueMap) {
                    if (key == _CALLBACK_EXTERNAL_CODE) {
                      if (onSuccess != null) onSuccess(new Device(valueMap));
                    }
                  })
                }
            })
        .catchError((error) => {
              if (error is PlatformException)
                {
                  if (onError != null)
                    onError(ErrorModel(error.code, error.message))
                }
            });
  }

  /// Set Log in with external Id
  ///
  /// externalId param [String] with the code
  /// Callback to get [Device] info
  /// Throw [ErrorModel] if there is some error during the processs
  static void logIn(String externalId, Function(Device device)? onSuccess,
      Function(ErrorModel error)? onError) async {
    _indigitall
        .invokeMapMethod(_ACTION_LOGIN, externalId)
        .then((value) => {
              if (value != null)
                {
                  value.forEach((key, valueMap) {
                    if (key == _CALLBACK_LOGIN) {
                      if (onSuccess != null) onSuccess(new Device(valueMap));
                    }
                  })
                }
            })
        .catchError((error) => {
              if (error is PlatformException)
                {
                  if (onError != null)
                    onError(ErrorModel(error.code, error.message))
                }
            });
  }

  /// Set Log out
  ///
  /// Callback to get [Device] info
  /// Throw [ErrorModel] if there is some error during the processs
  static void logOut(Function(Device device)? onSuccess,
      Function(ErrorModel error)? onError) async {
    _indigitall
        .invokeMapMethod(_ACTION_LOGOUT)
        .then((value) => {
              if (value != null)
                {
                  value.forEach((key, valueMap) {
                    if (key == _CALLBACK_LOGOUT) {
                      if (onSuccess != null) onSuccess(new Device(valueMap));
                    }
                  })
                }
            })
        .catchError((error) => {
              if (error is PlatformException)
                {
                  if (onError != null)
                    onError(ErrorModel(error.code, error.message))
                }
            });
  }

  /// Method to create a custom event that is working on backend
  ///
  /// Map params with eventCustom name and custom data.
  /// Callback to get info of custom event.
  /// Throw [ErrorModel] if there is some error during the processs
  static void sendCustomEvent(Map params, Function()? onSuccess,
      Function(ErrorModel error)? onError) async {
    _indigitall
        .invokeMapMethod(_ACTION_SENDCUSTOMEVENT, params)
        .then((value) => {
              if (value != null)
                {
                  value.forEach((key, valueMap) {
                    if (key == _CALLBACK_SEND_CUSTOM_EVENT) {
                      if (onSuccess != null) onSuccess();
                    }
                  })
                }
            })
        .catchError((error) => {
              if (error is PlatformException)
                {
                  if (onError != null)
                    onError(ErrorModel(error.code, error.message))
                }
            });
  }

  ///Action to get push in Android
  ///
  /// Callback to get the [Push] info when it is clicked
  /// Throw [ErrorModel] if there is some error during the processs
  static void getPush(Function(Push push)? onSuccess,
      Function(ErrorModel error)? onError) async {
    _indigitall
        .invokeMapMethod(_ACTION_GET_PUSH)
        .then((value) => {
              if (value != null)
                {
                  value.forEach((key, valueMap) {
                    if (key == _CALLBACK_GET_PUSH) {
                      if (onSuccess != null)
                        onSuccess(new Push(new Map<String, dynamic>.from(
                            json.decode(valueMap))));
                    }
                  })
                }
            })
        .catchError((error) => {
              if (error is PlatformException)
                {
                  if (onError != null)
                    onError(ErrorModel(error.code, error.message))
                }
            });
  }

  /// Get the a validation token for an application that has configured authentication with webhook
  ///
  /// Context param
  static void _getAuthentication(BuildContext context) async {
    if (context is InboxAuthListener) {
      Map? map = (context as InboxAuthListener?)?.getAuthConfig();
      await _indigitall.invokeMapMethod(_ACTION_GET_AUTH_CONFIG, map);
    }
  }

  /// Get [Inbox] for a external Id
  ///
  /// param [context] of Activity's context
  /// Callback to get [Inbox] on success
  /// Throw [ErrorModel] if there is some error during the processs
  static void getInbox(BuildContext context, Function(Inbox inbox)? onSuccess,
      Function(ErrorModel error)? onError) async {
    _getAuthentication(context);
    await _indigitall
        .invokeMapMethod(_ACTION_GET_INBOX)
        .then((value) => {
              if (value != null)
                {
                  value.forEach((key, valueMap) {
                    if (key == _CALLBACK_GET_INBOX) {
                      if (onSuccess != null) onSuccess(new Inbox(valueMap));
                    }
                  })
                }
            })
        .catchError((error) => {
              if (error is PlatformException)
                {
                  if (onError != null)
                    onError(ErrorModel(error.code, error.message))
                }
            });
  }

  /// Get the counters of Inbox
  ///
  /// param [context] of Activity's context
  /// Callback to get [InboxCounters] on success
  /// Throw [ErrorModel] if there is some error during the processs
  static void getMessageCount(
      BuildContext context,
      Function(InboxCounter counters)? onSuccess,
      Function(ErrorModel error)? onError) async {
    _getAuthentication(context);
    await _indigitall
        .invokeMapMethod(_ACTION_GET_MESSAGE_COUNT)
        .then((value) => {
              if (value != null)
                {
                  value.forEach((key, valueMap) {
                    if (key == _CALLBACK_GET_MESSAGE_COUNT) {
                      if (onSuccess != null)
                        onSuccess(new InboxCounter(valueMap));
                    }
                  })
                }
            })
        .catchError((error) => {
              if (error is PlatformException)
                {
                  if (onError != null)
                    onError(ErrorModel(error.code, error.message))
                }
            });
  }

  /// Get next page of Inbox
  ///
  /// param [context] of Activity's context
  /// Callback to get [Inbox] on success
  /// Throw [ErrorModel] if there is some error during the processs
  static void getNextPage(
      BuildContext context,
      Function(Inbox inbox)? onSuccess,
      Function(ErrorModel error)? onError) async {
    _getAuthentication(context);
    await _indigitall
        .invokeMapMethod(_ACTION_GET_NEXT_PAGE)
        .then((value) => {
              if (value != null)
                {
                  value.forEach((key, valueMap) {
                    if (key == _CALLBACK_GET_NEXT_PAGE) {
                      if (onSuccess != null) onSuccess(new Inbox(valueMap));
                    }
                  })
                }
            })
        .catchError((error) => {
              if (error is PlatformException)
                {
                  if (onError != null)
                    onError(ErrorModel(error.code, error.message))
                }
            });
  }

  /// Edit one or more notifications state with SendingId list
  ///
  /// param [context] of Activity's context
  /// para [Map] with the list os sendingId and new status
  /// Callback to get [InboxNotification] on success
  /// Throw [ErrorModel] if there is some error during the processs
  static void massiveEditNotificationsWithSendingIdsList(
      BuildContext context,
      Map params,
      Function()? onSuccess,
      Function(ErrorModel error)? onError) async {
    _getAuthentication(context);
    await _indigitall
        .invokeMapMethod(_ACTION_MASSIVE_EDIT, params)
        .then((value) => {
              if (value != null)
                {
                  value.forEach((key, valueMap) {
                    if (key == _CALLBACK_MASSIVE_EDIT) {
                      if (onSuccess != null) onSuccess();
                    }
                  })
                }
            })
        .catchError((error) => {
              if (error is PlatformException)
                {
                  if (onError != null)
                    onError(ErrorModel(error.code, error.message))
                }
            });
  }

  /// Get info of a notification with SendingId list
  ///
  /// param [context] of Activity's context
  /// para [Map] with the list os sendingId
  /// Callback to get [InboxNotification] on success
  /// Throw [ErrorModel] if there is some error during the processs
  static void getInfoFromNotificationWithSendingId(
      BuildContext context,
      Map param,
      Function(InboxNotification notification)? onSuccess,
      Function(ErrorModel error)? onError) async {
    _getAuthentication(context);
    await _indigitall
        .invokeMapMethod(_ACTION_GET_INFO_NOTIFICATION, param)
        .then((value) => {
              if (value != null)
                {
                  value.forEach((key, valueMap) {
                    if (key == _CALLBACK_GET_INFO_NOTIFICATION) {
                      if (Platform.isAndroid) {
                        if (onSuccess != null)
                          onSuccess(new InboxNotification(valueMap));
                      } else {
                        if (onSuccess != null)
                          onSuccess(new InboxNotification(
                              json.decode(valueMap.toString())));
                      }
                    }
                  })
                }
            })
        .catchError((error) => {
              if (error is PlatformException)
                {
                  if (onError != null)
                    onError(ErrorModel(error.code, error.message))
                }
            });
  }

  /// Edit one notification state with SeindingId list
  ///
  /// param [context] of Activity's context
  /// para [Map] with the sendingId and new status
  /// Callback to get [InboxNotification] on success
  /// Throw [ErrorModel] if there is some error during the processs
  static void modifyStatusFromNotificationWithSendingId(
      BuildContext context,
      Map params,
      Function(InboxNotification notification)? onSuccess,
      Function(ErrorModel error)? onError) async {
    _getAuthentication(context);
    await _indigitall
        .invokeMapMethod(_ACTION_MODIFY_STATUS, params)
        .then((value) => {
              if (value != null)
                {
                  value.forEach((key, valueMap) {
                    if (key == _CALLBACK_MODIFY_STATUS) {
                      if (Platform.isAndroid) {
                        if (onSuccess != null)
                          onSuccess(new InboxNotification(valueMap));
                      } else {
                        if (onSuccess != null)
                          onSuccess(new InboxNotification(
                              json.decode(valueMap.toString())));
                      }
                    }
                  })
                }
            })
        .catchError((error) => {
              if (error is PlatformException)
                {
                  if (onError != null)
                    onError(ErrorModel(error.code, error.message))
                }
            });
  }

  /// Show [InApp]
  ///
  /// Set a container and inApp code, if is correct it return a
  /// callback with inApp and container and return a container
  /// with the webView embeded
  static Future<Container?> showInApp(
      String inAppCode,
      Container container,
      Function(InApp inApp, Container container)? onSuccess,
      Function(ErrorModel error)? onError) async {
    return _getInApp(inAppCode, container, onSuccess, onError);
  }

  /// Show multiple [InApp]
  ///
  /// Set a list of containers and inApp codes, if is correct it return a
  /// callback with inApp and container and return a listView with all containers
  /// with the webViews embeded
  static Future<ListView> showMultipleInApp(
      List<String> inAppCodes,
      List<Container> containers,
      Function(InApp inApp, Container container)? onSuccess,
      Function(ErrorModel error)? onError) async {
    List<Container> listContainer = [];
    for (var i = 0; i < inAppCodes.length; i++) {
      var container =
          await _getInApp(inAppCodes[i], containers[i], onSuccess, onError);
      if (container != null) listContainer.add(container);
    }
    return ListView.builder(
        itemCount: listContainer.length,
        shrinkWrap: true, // <- added
        //primary: false, //
        itemBuilder: (BuildContext context, int index) {
          return listContainer[index];
        });
  }

  /// Get object [InApp]
  ///
  /// Set the vars to return a  container with a webView
  static Future<Container?> _getInApp(
      String inAppCode,
      Container container,
      Function(InApp inApp, Container container)? onSuccess,
      Function(ErrorModel error)? onError) async {
    await _indigitall
        .invokeMapMethod(_ACTION_GET_INAPP, inAppCode)
        .then((value) => {
              if (value != null)
                {
                  value.forEach((key, valueMap) {
                    if (key == _CALLBACK_GET_INAPP) {
                      InApp newInApp = new InApp(valueMap);
                      if (newInApp.schema != null &&
                          newInApp.schema!.height != null &&
                          newInApp.schema!.width != null) {
                        var height = container.constraints!.maxHeight;
                        var width = container.constraints!.maxWidth;
                        InAppWebViewController webView;
                        container = Container(
                            width: width,
                            height: height,
                            child: Expanded(
                                child: Stack(
                              children: [
                                InAppWebView(
                                  initialUrlRequest: URLRequest(
                                      url: Uri.parse(
                                          newInApp.properties!.contentUrl!)),
                                  onWebViewCreated: (controller) {
                                    webView = controller;
                                  },
                                ),
                                GestureDetector(
                                  behavior: HitTestBehavior.translucent,
                                  // onTapDown: (_) => print('webview onTapDown $_'),
                                  onTapUp: (_) async => {
                                    _indigitall.invokeMethod(
                                        _ACTION_TAP_INAPP, newInApp.toMap())
                                  },
                                  child: Container(),
                                ),
                              ],
                            )));
                        if (onSuccess != null) onSuccess(newInApp, container);
                      }
                    }
                  })
                }
            })
        .catchError((error) => {
              if (error is PlatformException)
                {
                  if (onError != null)
                    onError(ErrorModel(error.code, error.message))
                }
            });
    return container;
  }

  /// Show a pop up
  ///
  /// Map params with the InAppCode, the icon name you put in native modules
  /// and boolean icon disabled. Callbacks if [InApp] is load
  ///// clicked, closed, dismissed or showTime finished.
  /// Throw [ErrorModel] if there is some error during the processs
  static void showPopUp(
      Map params,
      Function(String inAppCode)? onLoad,
      // Function(String inAppCode, int time)? onShowTimeFinished,
      // Function()? didClicked,
      // Function()? didClosed,
      // Function()? didDismissed,
      Function(ErrorModel error)? onFail) async {
    _indigitall
        .invokeMapMethod(_ACTION_SHOW_POP_UP, params)
        .then((value) => {
              if (value != null)
                {
                  value.forEach((key, valueMap) {
                    switch (key) {
                      case _CALLBACK_SHOW_POPUP:
                        if (onLoad != null) onLoad(valueMap.toString());
                        break;
                      case _CALLBACK_DID_CLICKED:
                        //if (didClicked != null) didClicked();
                        break;
                      case _CALLBACK_DID_CLOSED:
                        //if (didClosed != null) didClosed();
                        break;
                      case _CALLBACK_DID_DISMISSED:
                        //if (didDismissed != null) didDismissed();
                        break;
                      case _CALLBACK_SHOWTIME_FINISHED:
                        Map map = valueMap;
                        String code = "";
                        int time = 0;
                        map.forEach((keyShowTime, valueShowtime) {
                          if (keyShowTime == "showtime")
                            time = int.parse(valueShowtime.toString());
                          if (keyShowTime == "inAppCode")
                            code = valueShowtime.toString();
                        });
                        // if (onShowTimeFinished != null)
                        //   onShowTimeFinished(code, time);
                        break;
                    }
                  })
                }
            })
        .catchError((error) => {
              if (error is PlatformException)
                {
                  if (onFail != null)
                    onFail(ErrorModel(error.code, error.message))
                }
            });
  }
}
