import 'dart:convert';

import 'package:indigitall_flutter_plugin/push/models/ExternalApp.dart';

class Device {
  static const String _MAP_ENABLE = "enable";
  static const String _MAP_DEVICE_ID = "deviceId";
  static const String _MAP_PUSH_TOKEN = "pushToken";
  static const String _MAP_PLATFORM = "platform";
  static const String _MAP_VERSION = "version";
  static const String _MAP_PRODUCT_NAME = "productName";
  static const String _MAP_PRODUCT_VERSION = "productVersion";
  static const String _MAP_OS_NAME = "osName";
  static const String _MAP_OS_VERSION = "osVersion";
  static const String _MAP_DEVICE_BRAND = "deviceBrand";
  static const String _MAP_DEVICE_MODEL = "deviceModel";
  static const String _MAP_OPERATOR = "operator";
  static const String _MAP_DEVICE_TYPE = "deviceType";
  static const String _MAP_APP_VERSION = "appVersion";
  static const String _MAP_LOCALE = "locale";
  static const String _MAP_TIME_ZONE = "timeZone";
  static const String _MAP_TIME_OFFSET = "timeOffset";
  static const String _MAP_EXTERNAL_APPS = "externalApps";
  static const String _MAP_EXTERNAL_CODE = "externalCode";

  String? deviceId;
  bool isEnabled = false;
  String? pushToken;
  String? platform;
  String? version;
  String? productName;
  String? productVersion;
  String? osName;
  String? osVersion;
  String? deviceBrand;
  String? deviceModel;
  String? operator;
  String? deviceType;
  String? appVersion;
  String? locale;
  String? timeZone;
  int? timeOffset;
  List<ExternalApp>? externalApps;
  String? externalCode;

  /// Constructo of [Device]
  Device(Map map) {
    map.forEach((key, value) {
      switch (key) {
        case _MAP_ENABLE:
          isEnabled = value.toString() == "true";
          break;
        case _MAP_DEVICE_ID:
          deviceId = value.toString();
          break;
        case _MAP_PUSH_TOKEN:
          pushToken = value.toString();
          break;
        case _MAP_PLATFORM:
          platform = value.toString();
          break;
        case _MAP_VERSION:
          version = value.toString();
          break;
        case _MAP_PRODUCT_NAME:
          productName = value.toString();
          break;
        case _MAP_PRODUCT_VERSION:
          productVersion = value.toString();
          break;
        case _MAP_OS_NAME:
          osName = value.toString();
          break;
        case _MAP_OS_VERSION:
          osVersion = value.toString();
          break;
        case _MAP_DEVICE_BRAND:
          deviceBrand = value.toString();
          break;
        case _MAP_DEVICE_MODEL:
          deviceModel = value.toString();
          break;
        case _MAP_OPERATOR:
          operator = value.toString();
          break;
        case _MAP_DEVICE_TYPE:
          deviceType = value.toString();
          break;
        case _MAP_APP_VERSION:
          appVersion = value.toString();
          break;
        case _MAP_LOCALE:
          locale = value.toString();
          break;
        case _MAP_TIME_ZONE:
          timeZone = value.toString();
          break;
        case _MAP_TIME_OFFSET:
          timeOffset = int.parse(value.toString());
          break;
        case _MAP_EXTERNAL_CODE:
          externalCode = value.toString();
          break;
        case _MAP_EXTERNAL_APPS:
          {
            String externalAppsString = value.toString();
            externalApps = ExternalApp.createArray(externalAppsString);
          }
          break;
      }
    });
  }

  /// conversion to [Map]
  Map toMap() {
    Map map = Map();
    if (deviceId != null) map[_MAP_DEVICE_ID] = deviceId;
    map[_MAP_ENABLE] = isEnabled;
    if (pushToken != null) map[_MAP_PUSH_TOKEN] = pushToken;
    if (platform != null) map[_MAP_PLATFORM] = platform;
    if (version != null) map[_MAP_VERSION] = version;
    if (productName != null) map[_MAP_PRODUCT_NAME] = productName;
    if (productVersion != null) map[_MAP_PRODUCT_VERSION] = productVersion;
    if (osName != null) map[_MAP_OS_NAME] = osName;
    if (osVersion != null) map[_MAP_OS_VERSION] = osVersion;
    if (deviceBrand != null) map[_MAP_DEVICE_BRAND] = deviceBrand;
    if (deviceModel != null) map[_MAP_DEVICE_MODEL] = deviceModel;
    if (operator != null) map[_MAP_OPERATOR] = operator;
    if (deviceType != null) map[_MAP_DEVICE_TYPE] = deviceType;
    if (appVersion != null) map[_MAP_APP_VERSION] = appVersion;
    if (locale != null) map[_MAP_LOCALE] = locale;
    if (timeZone != null) map[_MAP_TIME_ZONE] = timeZone;
    if (timeOffset != null) map[_MAP_TIME_OFFSET] = timeOffset;
    if (externalApps != null) {
      List<Map> list = [];
      for (var i = 0; i < externalApps!.length; i++) {
        list.add(externalApps![i].toMap());
      }
      map[_MAP_EXTERNAL_APPS] = list;
    }
    if (externalCode != null) map[_MAP_EXTERNAL_CODE] = externalCode;
    return map;
  }

  /// Conversion to [String]
  @override
  String toString() {
    return json.encode(toMap());
  }
}
